@@ -1,19 +1,3 @@
-"""
-Copyright Astronomer, Inc.
-
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-"""
-
 from typing import Callable, Optional
 
 from airflow.decorators.base import task_decorator_factory
@@ -28,19 +12,25 @@ def dataframe(
     database: Optional[str] = None,
     schema: Optional[str] = None,
     warehouse: Optional[str] = None,
+    task_id: Optional[str] = None,
+    identifiers_as_lower: Optional[bool] = True,
 ):
     """
     This function allows a user to run python functions in Airflow but with the huge benefit that SQL files
     will automatically be turned into dataframes and resulting dataframes can automatically used in astro.sql functions
     """
+    param_map = {
+        "conn_id": conn_id,
+        "database": database,
+        "schema": schema,
+        "warehouse": warehouse,
+        "identifiers_as_lower": identifiers_as_lower,
+    }
+    if task_id:
+        param_map["task_id"] = task_id
     return task_decorator_factory(
         python_callable=python_callable,
         multiple_outputs=multiple_outputs,
         decorated_operator_class=SqlDataframeOperator,  # type: ignore
-        **{
-            "conn_id": conn_id,
-            "database": database,
-            "schema": schema,
-            "warehouse": warehouse,
-        }
+        **param_map,
     )
