@@ -1,47 +1,34 @@
-"""
-Copyright Astronomer, Inc.
-
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-"""
 import inspect
-import os
-from builtins import NotImplementedError
 from typing import Callable, Dict, Iterable, Mapping, Optional, Union
 
 import pandas as pd
 from airflow.decorators.base import DecoratedOperator, task_decorator_factory
 from airflow.hooks.base import BaseHook
-from airflow.hooks.sqlite_hook import SqliteHook
-from airflow.models import DagRun, TaskInstance
 from airflow.utils.db import provide_session
-from sqlalchemy import text
+from sqlalchemy.exc import ProgrammingError
 from sqlalchemy.sql.functions import Function
 
-from astro.sql.table import Table, TempTable, create_table_name
-from astro.utils import postgres_transform, snowflake_transform
-from astro.utils.dependencies import BigQueryHook, PostgresHook, SnowflakeHook
-from astro.utils.load_dataframe import move_dataframe_to_sql
-from astro.utils.schema_util import create_schema_query, get_schema, schema_exists
+from astro.constants import Database
+from astro.settings import SCHEMA
+from astro.sql.table import Table, TempTable, create_unique_table_name
+from astro.utils import get_hook, postgres_transform, snowflake_transform
+from astro.utils.database import (
+    get_database_from_conn_id,
+    get_sqlalchemy_engine,
+    run_sql,
+)
+from astro.utils.load import load_dataframe_into_sql_table
+from astro.utils.schema_util import create_schema_query, schema_exists
 from astro.utils.table_handler import TableHandler
 
 
-class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
+class SqlDecoratedOperator(DecoratedOperator, TableHandler):
     def __init__(
         self,
         conn_id: Optional[str] = None,
         autocommit: bool = False,
-        parameters: dict = None,
-        handler: Function = None,
+        parameters: Optional[dict] = None,
+        handler: Optional[Function] = None,
         database: Optional[str] = None,
         schema: Optional[str] = None,
         warehouse: Optional[str] = None,
@@ -50,24 +37,6 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
         sql="",
         **kwargs,
     ):
-        """
-        :param to_dataframe: This function allows users to pull the current staging table into a pandas dataframe.
-
-            To use this function, please make sure that your decorated function has a parameter called ``input_df``. This
-        parameter will be a pandas.Dataframe that you can modify as needed. Please note that until we implement
-        spark and dask dataframes, that you should be mindful as to how large your worker is when pulling large tables.
-        :param from_s3: Whether to pull from s3 into current database.
-
-            When set to true, please include a parameter named ``s3_path`` in your TaskFlow function. When calling this
-        task, you can specify any s3:// path and Airflow will
-        automatically pull that file into your database using Panda's automatic data typing functionality.
-        :param from_csv: Whether to pull from a local csv file into current database.
-
-            When set to true, please include a parameter named ``csv_path`` in your TaskFlow function.
-        When calling this task, you can specify any local path and Airflow will automatically pull that file into your
-        database using Panda's automatic data typing functionality.
-        :param kwargs:
-        """
         self.raw_sql = raw_sql
         self.autocommit = autocommit
         self.parameters = parameters
@@ -80,6 +49,9 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
         else:
             self.output_table = None
 
+        if self.op_kwargs.get("handler"):
+            self.handler = self.op_kwargs.pop("handler")
+
         self.database = self.op_kwargs.pop("database", database)
         self.conn_id = self.op_kwargs.pop("conn_id", conn_id)
         self.schema = self.op_kwargs.pop("schema", schema)
@@ -90,23 +62,33 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
             **kwargs,
         )
 
-    def execute(self, context: Dict):
+    @property
+    def conn_type(self):
+        return BaseHook.get_connection(self.conn_id).conn_type
 
+    @property
+    def hook(self):
+        return get_hook(
+            conn_id=self.conn_id,
+            database=self.database,
+            role=self.role,
+            schema=self.schema,
+            warehouse=self.warehouse,
+        )
+
+    def execute(self, context: Dict):
         if not isinstance(self.sql, str):
-            cursor = self._run_sql_alchemy_obj(self.sql, self.parameters)
+            cursor = self._run_sql(self.sql, self.parameters)
             if self.handler is not None:
                 return self.handler(cursor)
             return cursor
 
-        self.output_schema = self.schema or get_schema()
         self._set_variables_from_first_table()
 
         conn = BaseHook.get_connection(self.conn_id)
-        self.conn_type = conn.conn_type  # type: ignore
-        self.schema = self.schema or get_schema()
+
+        self.schema = self.schema or SCHEMA
         self.user = conn.login
-        self.run_id = context.get("run_id")
-        self._set_hook()
 
         self.convert_op_arg_dataframes()
         self.convert_op_kwarg_dataframes()
@@ -118,6 +100,8 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
         self._process_params()
 
         output_table_name: str = ""
+        # Create DEFAULT SCHEMA if not created.
+        self._set_schema_if_needed(SCHEMA)
 
         if not self.raw_sql:
             # Create a table name for the temp table
@@ -127,11 +111,11 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
                 self._set_schema_if_needed()
 
             if not self.output_table or type(self.output_table) == TempTable:
-                output_table_name = create_table_name(context=context)
+                output_table_name = create_unique_table_name()
                 full_output_table_name = self.handle_output_table_schema(
                     # Since there is no output table defined we have to assume default schema
                     output_table_name,
-                    schema=get_schema(),
+                    schema=SCHEMA,
                 )
             else:
                 output_table_name = self.output_table.table_name
@@ -139,12 +123,16 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
                     output_table_name, schema=self.output_table.schema
                 )
 
-            self._run_sql_alchemy_obj(
+            self._run_sql(
                 f"DROP TABLE IF EXISTS {full_output_table_name};", self.parameters
             )
             self.sql = self.create_temporary_table(self.sql, full_output_table_name)
+        else:
+            # If there's no SQL to run we simply return
+            if self.sql == "" or not self.sql:
+                return
 
-        query_result = self._run_sql_alchemy_obj(self.sql, self.parameters)
+        query_result = self._run_sql(self.sql, self.parameters)
         # Run execute function of subclassed Operator.
 
         if self.output_table:
@@ -152,7 +140,7 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
                 self.output_table = self.output_table.to_table(
                     table_name=output_table_name, schema=self.output_table.schema
                 )
-            self.log.info(f"returning table {self.output_table}")
+            self.log.info("Returning table %s", self.output_table)
             self.populate_output_table()
             return self.output_table
 
@@ -165,7 +153,7 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
                 table_name=output_table_name,
             )
             self.populate_output_table()
-            self.log.info(f"returning table {self.output_table}")
+            self.log.info("Returning table %s", self.output_table)
             return self.output_table
 
     def read_sql(self):
@@ -182,7 +170,6 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
                 self.sql = file.read().replace("\n", " ")
 
     def handle_params(self, context):
-        # Automatically add any kwargs going into the function
         if self.op_kwargs:
             self.parameters.update(self.op_kwargs)  # type: ignore
         if self.op_args:
@@ -195,133 +182,55 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
             }
         self.parameters.update(self.op_kwargs)  # type: ignore
 
-    def handle_output_table_schema(self, output_table_name, schema=None):
+    def handle_output_table_schema(
+        self, output_table_name: str, schema: Optional[str] = None
+    ) -> str:
         """
         In postgres, we set the schema in the query itself instead of as a query parameter.
         This function adds the necessary {schema}.{table} notation.
-        :param output_table_name:
+
+        :param output_table_name: Output table name
         :param schema: an optional schema if the output_table has a schema set. Defaults to the temp schema
-        :return:
         """
-        schema = schema or get_schema()
-        if self.conn_type == "postgres" and self.schema:
+        schema = schema or SCHEMA
+        database = get_database_from_conn_id(self.conn_id)
+        if database in (Database.POSTGRES, Database.BIGQUERY) and self.schema:
             output_table_name = schema + "." + output_table_name
-        elif self.conn_type == "snowflake" and self.schema and "." not in self.sql:
+        elif database == Database.SNOWFLAKE and self.schema and "." not in self.sql:
             output_table_name = self.database + "." + schema + "." + output_table_name
         return output_table_name
 
-    def _set_hook(self):
-        if self.conn_type == "postgres":
-            self.hook = PostgresHook(
-                postgres_conn_id=self.conn_id, schema=self.database
-            )
-
-        elif self.conn_type == "snowflake":
-            self.hook = self.get_snow_hook()
-
-    def _set_schema_if_needed(self):
-        schema_statement = ""
-        if self.conn_type == "postgres":
+    def _set_schema_if_needed(self, schema=None):
+        if self.conn_type in ["postgres", "snowflake", "bigquery"]:
             schema_statement = create_schema_query(
                 conn_type=self.conn_type,
                 hook=self.hook,
-                schema_id=self.schema,
+                schema_id=schema if schema else self.schema,
                 user=self.user,
             )
-        elif self.conn_type == "snowflake":
-            schema_statement = create_schema_query(
-                conn_type=self.conn_type,
-                hook=self.hook,
-                schema_id=self.schema,
-                user=self.user,
-            )
-        self._run_sql_string(schema_statement, {})
-
-    def get_snow_hook(self) -> SnowflakeHook:
-        """
-        Create and return SnowflakeHook.
-        :return: a SnowflakeHook instance.
-        :rtype: SnowflakeHook
-        """
-        return SnowflakeHook(
-            snowflake_conn_id=self.conn_id,
-            warehouse=self.warehouse,
-            database=self.database,
-            role=self.role,
-            schema=self.schema,
-            authenticator=None,
-            session_parameters=None,
-        )
-
-    def get_bigquery_hook(self):
-        return BigQueryHook(
-            use_legacy_sql=False,
-            gcp_conn_id=self.conn_id,
-        )
-
-    def get_sqlite_hook(self):
-        return SqliteHook(
-            sqlite_conn_id=self.conn_id,
-        )
-
-    def get_postgres_hook(self):
-        return PostgresHook(postgres_conn_id=self.conn_id, schema=self.database)
-
-    def _run_sql_string(self, sql, parameters):
-        if self.conn_type == "postgres":
-            self.log.info("Executing: %s", sql)
-            self.hook = PostgresHook(
-                postgres_conn_id=self.conn_id, schema=self.database
-            )
-            results = self.hook.run(
-                sql,
-                self.autocommit,
-                parameters=parameters,
-                handler=self.handler,
-            )
-            for output in self.hook.conn.notices:
-                self.log.info(output)
-
-        elif self.conn_type == "snowflake":
-            self.log.info("Executing: %s", sql)
-            hook = self.get_snow_hook()
-            results = hook.run(sql, autocommit=self.autocommit, parameters=parameters)
-            self.query_ids = hook.query_ids
-
-        elif self.conn_type == "bigquery":
-            hook = self.get_bigquery_hook()
-            results = hook.run(sql, autocommit=self.autocommit, parameters=parameters)
-        return results
+            try:
+                self._run_sql(schema_statement, {})
+            except ProgrammingError as e:
+                print(e)
 
     def get_sql_alchemy_engine(self):
-        conn = BaseHook.get_connection(self.conn_id)
-        self.conn_type = conn.conn_type  # type: ignore
+        return get_sqlalchemy_engine(self.hook)
 
-        hook = {
-            "snowflake": self.get_snow_hook,
-            "postgresql": self.get_postgres_hook,
-            "postgres": self.get_postgres_hook,
-            "bigquery": self.get_bigquery_hook,
-            "sqlite": self.get_sqlite_hook,
-        }[self.conn_type]()
-        return hook.get_sqlalchemy_engine()
+    def _run_sql(self, sql, parameters=None):
+        return run_sql(
+            engine=self.get_sql_alchemy_engine(),
+            sql_statement=sql,
+            parameters=parameters,
+        )
 
-    def _run_sql_alchemy_obj(self, sql, parameters):
-        engine = self.get_sql_alchemy_engine()
-        conn = engine.connect()
-        if isinstance(sql, str):
-            return conn.execute(text(sql), parameters)
-        else:
-            return conn.execute(sql, parameters)
-
-    @staticmethod
-    def create_temporary_table(query, output_table_name, schema=None):
+    def create_temporary_table(self, query, output_table_name, schema=None):
         """
-        Create a temp table for the current task instance. This table will be overwritten if the DAG is run again as this
-        table is only ever meant to be temporary.
-        :param query:
-        :param table_name
-        :return:
+        Create a temp table for the current task instance. This table will be overwritten if
+        the DAG is run again as this table is only ever meant to be temporary.
+
+        :param query: The Query to run
+        :param output_table_name: The name of the table to create
+        :param schema: The schema to create the table in
         """
 
         def clean_trailing_semicolon(query):
@@ -332,22 +241,12 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
 
         if schema:
             output_table_name = f"{schema}.{output_table_name}"
-        return (
-            f"CREATE TABLE {output_table_name} AS ({clean_trailing_semicolon(query)});"
-        )
 
-    @staticmethod
-    def create_cte(query, table_name):
-        return f"WITH {table_name} AS ({query}) SELECT * FROM {table_name};"
-
-    @staticmethod
-    def create_output_csv_path(context):
-        ti: TaskInstance = context["ti"]
-        dag_run: DagRun = ti.get_dagrun()
-        return f"{dag_run.dag_id}_{ti.task_id}_{int(ti.execution_date.timestamp())}.csv"
-
-    def handle_dataframe_func(self, input_table):
-        raise NotImplementedError("Need to add dataframe func to class")
+        if self.conn_type == "sqlite":
+            statement = f"CREATE TABLE {output_table_name} AS {clean_trailing_semicolon(query)};"
+        else:
+            statement = f"CREATE TABLE {output_table_name} AS ({clean_trailing_semicolon(query)});"
+        return statement
 
     @provide_session
     def pre_execute(self, context, session=None):
@@ -360,10 +259,6 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
         """
         pass
 
-    def _table_exists_in_db(self, conn: str, table_name: str):
-        """Override this method to enable sensing db."""
-        raise NotImplementedError("Add _table_exists_in_db method to class")
-
     def _process_params(self):
         if self.conn_type == "snowflake":
             self.parameters = snowflake_transform.process_params(
@@ -371,9 +266,10 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
             )
 
     def _add_templates_to_context(self, context):
-        if self.conn_type in ["postgres", "bigquery"]:
+        database = get_database_from_conn_id(self.conn_id)
+        if database in (Database.POSTGRES, Database.BIGQUERY):
             return postgres_transform.add_templates_to_context(self.parameters, context)
-        elif self.conn_type in ["snowflake"]:
+        elif database == Database.SNOWFLAKE:
             return snowflake_transform.add_templates_to_context(
                 self.parameters, context
             )
@@ -382,7 +278,7 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
 
     def default_transform(self, parameters, context):
         for k, v in parameters.items():
-            if type(v) == Table:
+            if isinstance(v, Table):
                 context[k] = v.table_name
             else:
                 context[k] = ":" + k
@@ -397,55 +293,50 @@ class SqlDecoratoratedOperator(DecoratedOperator, TableHandler):
         final_args = []
         for i, arg in enumerate(self.op_args):
             if type(arg) == pd.DataFrame:
-                output_table_name = (
-                    self.dag_id
-                    + "_"
-                    + self.task_id
-                    + "_"
-                    + self.run_id
-                    + f"_input_dataframe_{i}"
-                )
-                move_dataframe_to_sql(
-                    output_table_name=output_table_name,
-                    df=arg,
-                    conn_type=self.conn_type,
+                pandas_dataframe = arg
+                output_table_name = create_unique_table_name()
+                output_table = Table(
+                    table_name=output_table_name,
                     conn_id=self.conn_id,
                     database=self.database,
                     schema=self.schema,
                     warehouse=self.warehouse,
                 )
-                final_args.append(
-                    Table(
-                        table_name=output_table_name,
-                        conn_id=self.conn_id,
-                        database=self.database,
-                        schema=self.schema,
-                        warehouse=self.warehouse,
-                    )
+                hook = get_hook(
+                    conn_id=self.conn_id,
+                    database=self.database,
+                    schema=self.schema,
+                    warehouse=self.warehouse,
                 )
+                load_dataframe_into_sql_table(pandas_dataframe, output_table, hook)
+                final_args.append(output_table)
             else:
                 final_args.append(arg)
             self.op_args = tuple(final_args)
 
     def convert_op_kwarg_dataframes(self):
         final_kwargs = {}
-        for k, v in self.op_kwargs.items():
-            if type(v) == pd.DataFrame:
-                output_table_name = "_".join(
-                    [self.dag_id, self.task_id, self.run_id, "input_dataframe", str(k)]
-                )
-                move_dataframe_to_sql(
-                    output_table_name=output_table_name,
-                    df=v,
-                    conn_type=self.conn_type,
+        for key, value in self.op_kwargs.items():
+            if type(value) == pd.DataFrame:
+                pandas_dataframe = value
+                output_table_name = create_unique_table_name()
+                output_table = Table(
+                    table_name=output_table_name,
                     conn_id=self.conn_id,
                     database=self.database,
                     schema=self.schema,
                     warehouse=self.warehouse,
                 )
-                final_kwargs[k] = output_table_name
+                hook = get_hook(
+                    conn_id=self.conn_id,
+                    database=self.database,
+                    schema=self.schema,
+                    warehouse=self.warehouse,
+                )
+                load_dataframe_into_sql_table(pandas_dataframe, output_table, hook)
+                final_kwargs[key] = output_table
             else:
-                final_kwargs[k] = v
+                final_kwargs[key] = value
         self.op_kwargs = final_kwargs
 
 
@@ -469,7 +360,7 @@ def _transform_task(
     return task_decorator_factory(
         python_callable=python_callable,
         multiple_outputs=multiple_outputs,
-        decorated_operator_class=SqlDecoratoratedOperator,  # type: ignore
+        decorated_operator_class=SqlDecoratedOperator,  # type: ignore
         **kwargs,
     )
 
@@ -484,7 +375,7 @@ def transform_decorator(
     schema: Optional[str] = None,
     warehouse: Optional[str] = None,
     raw_sql: bool = False,
-    handler: Callable = None,
+    handler: Optional[Callable] = None,
 ):
     """
     :param python_callable:
